#include <avr/io.h>
#include <stdarg.h>
#include <string.h>
#include <avr/pgmspace.h>
#include "mydefs.h"

#include "lcd.h"
struct display lcd;
//---------------------------------------------------------------------------//
//---------------------------------------------------------------------------//
//---------------------------------------------------------------------------//
//---------------------------------------------------------------------------//
//---------------------------------------------------------------------------//
void lcd_write_dat (uint8_t dat)
{
	LCD_CTRL |=  _BV(LCD_RS);			//RS before High is Data
	LCD_CTRL &=  ~_BV(LCD_WR) | _BV(LCD_E);
	LCD_PORT = dat;
	//_delay_us (5);
	LCD_CTRL |= _BV(LCD_WR) | _BV(LCD_E);
}
//---------------------------------------------------------------------------//
void lcd_write_cmd (uint8_t cmd)
{
	LCD_CTRL &=  ~_BV(LCD_RS);		//Low is Command
	LCD_CTRL &=  ~_BV(LCD_WR) | _BV(LCD_E);
	LCD_PORT = cmd;
	//_delay_us (5);
	LCD_CTRL |= _BV(LCD_WR) | _BV(LCD_E);
}
//---------------------------------------------------------------------------//
void clear_screen (char fill)
{
    uint16_t	 r, c;

	for(r = 0;r < 8;r++)
	{
    	lcd_write_cmd(START_C);			//set lower column
	    lcd_write_cmd(0x10);			//set higher column
	    lcd_write_cmd(SET_PAGE + r);	//set page
		for(c = 0;c < 128; c ++)
		{
			lcd_write_dat(fill);		//0x00 clear
		}
	}
}
//---------------------------------------------------------------------------//
void lcd_gotoxy (uint8_t x, uint8_t y)
{
	lcd.x = x; lcd.y = y;
	lcd_write_cmd(0x00 + (x % 16));
	lcd_write_cmd(0x10 + (x / 16));
	lcd_write_cmd(SET_PAGE + (y/8));	//set page	
}
//---------------------------------------------------------------------------//
void lcd_NewLine(uint8_t fontHeight, uint8_t offset) 
{
	if (lcd.y + fontHeight < 64)
		lcd_gotoxy(offset, lcd.y + fontHeight);
	else
		lcd_gotoxy(offset, 0);
}
//---------------------------------------------------------------------------//
//---------------------------------------------------------------------------//
//---------------------------------------------------------------------------//
//---------------------------------------------------------------------------//
//---------------------------------------------------------------------------//
//---------------------------------------------------------------------------//
//------------------------------------------------------------------------//
void glcd_PutChar(char c, struct font font) 
{
	uint16_t index;
	uint8_t pages, tmp, i, j, xPos, yPos, xtmp;
	
	if(c == '\n')
		lcd_NewLine(font.height, 0);
	if(c < 32)										// ignore escape characters
		return;
	
	xPos = lcd.x;									// save old coordinates
	yPos = lcd.y;
	c -= 32;
	
	pages = font.height/8;							// calculate pages
													// Small Font = 0;
													// BIG FONT	  = 1 ->upper;
	if(font.height%8 != 0)
		pages++;
	
	index = c * font.width * pages;					// get the needed array index

	for(j = 0; j < pages; j++) 
	{
		xtmp = xPos;
		
		for(i=j; i< font.width * pages; i += pages) 
		{
			tmp = pgm_read_byte(font.size + index + i);
			if (lcd.inv)							// NORMAL = 0, INV = 1;
				  tmp =~ tmp;						// Convert Data
			lcd_gotoxy (xtmp++,lcd.y);
			lcd_write_dat(tmp);					// write Character-Data
		}
		lcd_gotoxy(xPos, lcd.y + 8);
	}
	lcd_gotoxy(lcd.x + font.width, yPos);			// go to the upper right corner
}
//---------------------------------------------------------------------------//
void glcd_PutString(char *string, struct font font) 
{
	uint8_t startx =lcd.x, i=0;
	
	char c = string[0];
	
	while(c != 0) 
	{
		if(c == '\f')
			clear_screen(0);	
		if(c == '\n')
			lcd_NewLine(font.height, startx);
		else
			glcd_PutChar(c, font);
		c = string[++i];
	}
}
//---------------------------------------------------------------------------//
void glcd_Printf (uint8_t x, uint8_t y, char *string, uint8_t inv, uint8_t type)
{
		struct font 	largeFont, 
						smallFont;
	
		lcd.inv 			= inv;
				
		largeFont.width 	= FONT8X16_WIDTH;
		largeFont.height 	= FONT8X16_HEIGHT;
		largeFont.size 		= Font8x16;
	
		smallFont.width 	= FONT6X8_WIDTH;
		smallFont.height 	= FONT6X8_HEIGHT;
		smallFont.size	 	= Font6x8;
		
		lcd_gotoxy (x,y);
		if (!type)
			glcd_PutString (string, smallFont);
		else
			glcd_PutString (string, largeFont);	
}
//---------------------------------------------------------------------------//
void init_lcd (void)
{
	LCD_DDR			|=  0xff; 			    //all port output
	LCD_CTRL_DDR	|= _BV(LCD_RD)  | _BV(LCD_WR)   | _BV(LCD_RS) | _BV(LCD_E);//all port are output
	CTRL_DDR		|= _BV(LCD_RES) | _BV(LCD_DISP);
	
	CTRL_PORT		|= _BV(LCD_DISP);		// active high for display
	CTRL_PORT 		|= _BV(LCD_RES);		// Reset high
	_delay_ms   	(10);
	CTRL_PORT		&= ~_BV(LCD_RES);		// active low
	_delay_ms 		(10);
	CTRL_PORT 		|= _BV(LCD_RES);		// active high
	_delay_ms 		(10);
	LCD_CTRL 		&= ~_BV(LCD_RD);		//active low
	

// jnut config , If you need to change something please see document
	
	
//(1)	
	lcd_write_cmd (0xAE);				//set dsiplay OFF
//(2)	
	lcd_write_cmd (0x26);				//horizontal scroll up
	lcd_write_cmd (0x01);				//4 byte  
	lcd_write_cmd (0x00);
	lcd_write_cmd (0x02);
	lcd_write_cmd (0x07);
//(3)
	lcd_write_cmd (0x81);				//set contrast
	lcd_write_cmd (0xff);				//register 256 step 00 - FF
//(4)
	lcd_write_cmd (0xDB);				//Set VCOM
	lcd_write_cmd (0x35);
//(5)	
	lcd_write_cmd (SET_SEGMENT);		//Set  Segment
	lcd_write_cmd (SET_COMOUT);			//set  comout scan direction
//(6)	
	lcd_write_cmd (0xAF);				//set display ON
	
}
//---------------------------------------------------------------------------//
