#include 	<avr/io.h>
#include 	<avr/interrupt.h>
#include 	<stdio.h>
#include 	<stdarg.h>
#include 	<string.h>
#include 	<stdlib.h>

#include 	"lcd.h"
#include 	"text.h"

#include 	"mydefs.h"
#include 	"serial.h"
#include 	"printf.h"

uint8_t 		lcdbuf[25];

double  		adc_value;
double 		Vref = 3.3; // Reference voltage Vref of ADC
							// Voltage Supply

#define 		ADC_ENABLE ADCSRA |= (1<<ADEN)
#define 		ADC_DISABLE ADCSRA &= 0x7F
#define 		ADC_START_CONVERSION ADCSRA |= (1<<ADSC)

//------------------------------------------------------------------------//
void delay (uint16_t n)
{
	for (;n > 0;n--)
		_delay_ms (200);
}
//------------------------------------------------------------------------//
void init_adc (uint8_t channel)
{
	ADCSRA 	= 0x00; 			//disable adc
	ADMUX 	= 0x40 | channel; 	//select adc input 
	ADCSRA 	= 0x86;
}
//------------------------------------------------------------------------//

// Read the AD conversion result
uint16_t read_adc(uint8_t channel)
{
	uint16_t ADC_temp, ADCH_temp;
	uint16_t ADC_var = 0;
	
	ADC_ENABLE;
		ADC_START_CONVERSION; 		//do a dummy readout first
		while(!(ADCSRA & 0x10)); 	// wait for conversion done, ADIF flag active
	ADCSRA|=(1<<ADIF);
	
	ADC_temp  =  ADCL; 				// read out ADCL register
	ADCH_temp =  ADCH; 				// read out ADCH register
	ADC_temp  += (ADCH_temp << 8);
	ADC_var   += ADC_temp; 			// accumulate result (8 samples)

			//for later averaging
			ADC_var = ADC_var >> 2; // average the 8 samples
		ADC_DISABLE;
	return ADC_var;
}
//------------------------------------------------------------------------//
//------------------------------------------------------------------------//
//------------------------------------------------------------------------//
void display_bar (uint16_t value)
{
	uint8_t i,dat;
		
		lcd_gotoxy (2,24);
		lcd_write_dat (0xff);
		value /=2;

			//if (value == 0)
			//	value +=1;
			for (i = 0; i < 126;i++)
			{
				if (i < value)
				{
					dat = 0x55;
					if (i%2) 
						dat = 0xAA;
					dat |= 0x81;
					lcd_write_dat (dat);
				}
				else
					lcd_write_dat (0x81);
			}
			lcd_write_dat (0xFF);
		for (i = 0;i < 128;i++)
		{
			lcd_gotoxy (i+2,32); lcd_write_dat (0x00);
		}
		lcd_gotoxy (1 + value + 1,32);
		lcd_write_dat (0xFF);
}
//------------------------------------------------------------------------//
int main(void)
{	
	uint16_t	adc;
	uint8_t		nCounter;
	
	ser_init();							//init serial port

	init_lcd ();						//init OLED
	
	init_adc (0);						//init ADC
	
	sei();								//enable interrupt
	
	clear_screen (0);
		
	glcd_Printf (2,0,   "http://www.se-ed.com ",NORMAL,SMALL_FONT);
	glcd_Printf (2,8,   "1234567890ABCDEFGHIJ ",NORMAL,SMALL_FONT);
	glcd_Printf (2,16,  "KLMNOPQRSTUVWXYZ...! ",NORMAL,SMALL_FONT);
	glcd_Printf (2,24,  "---------------------",NORMAL,SMALL_FONT);
	glcd_Printf (2,32,  "BIG FONT DEMO..!",NORMAL,BIG_FONT);
	glcd_Printf (2,48,  "jnutthailand.com",NORMAL,BIG_FONT);
		
	delay (50);
	
	clear_screen (0);
	
	for (nCounter = 0; nCounter < 99;nCounter++)
	{
		sprintf (lcdbuf,"%02d",nCounter);
		glcd_Printf (56,24,  lcdbuf,NORMAL,BIG_FONT);
		delay (1);
		
	}
	clear_screen (0);
	
	glcd_Printf (2,0,  "ATmega32 + OLED",NORMAL,BIG_FONT);
	
	for (;;)
	{

			adc = read_adc (0);
			adc_value = (adc * Vref/256.0); 
			memset (lcdbuf, 0, sizeof lcdbuf);
			display_bar (adc);
			//sprintf (lcdbuf,"ADC = %02X %0.02f",adc,adc_value);
			sprintf (lcdbuf,"Voltage = %0.02fV",adc_value);
			glcd_Printf (2,40,lcdbuf,NORMAL,BIG_FONT);
				
			delay (10);
	}

	while (1);
}
